/* Based off main.c for Lab #1
 * Nancy Minderman 
 * April 20, 2007
 */

/* This is going to be code pertaining to the motions sensors
 * Functions:
 *	CalculateNextBallLocation
 *	GetSensorNumber
 *	GetSensorStatusA
 *	GetSensorStatusB
 *	SendInstructionsToFPGA
 */

#include "motionSensors.h"

extern void at91_irq_handler(void);
extern void wake_up_handler (void) ;

/* Global Variable */
WaitDesc wait_desc = { &TC0_DESC, 0, 0, WAIT_DELAY, wake_up_handler } ;
int player1Score = 0;
int player2Score = 0;
TerminalDataDesc serialData;
TerminalDesc serial;

//*----------------------------------------------------------------------------
//* Function Name       : main
//* Object              : Main function of the motion sensor read.
//* Input Parameters    : none
//* Output Parameters   : True
//*----------------------------------------------------------------------------
int main( void )
//* Begin
{   
    //* -- Variables we need
    u_char ballDirectionX = LEFT;	// Starting x direction of the ball
    u_char ballDirectionY = DOWN;	// Starting y direction of the ball
    u_char ballSpeedX = 1;		// The ball's horizontal speed
    u_char ballSpeedY = 1;		// The ball's vertical speed
    u_char hit = HIT_NOTHING;		// Keeps track of what the ball hits

    u_char player1_AI = AI_OFF;		// Keeps track of the AI status for player 1
    u_char player2_AI = AI_OFF;		// Keeps track of the AI status for player 2
    u_char pause = NOT_PAUSED;		// Keeps track of whether or not the game is paused

    u_int currentSensorsA = 0;		// Used to find triggered sensors for player 1
    u_int currentSensorsB = 0;		// Used to find triggered sensors for player 2
    u_int speed = MIN_SPEED;		// Controls the speed of the ball (how often it is drawn in one place)
    u_int speedCounter = 0;		// Controls the speed of the ball (how often it is drawn in one place)
    u_int topMostSensorA = 0;		// Used to find position of left paddle
    u_int topMostSensorB = 0;		// Used to find position of right paddle
    u_int buttonStatusA = 0;		// Used to poll for button presses
    u_int buttonStatusB = 0;		// Used to poll for button presses
    u_int buttonWait = 0;		// Delay between button presses
    u_int startScreenCounter = 0;	// Used to display the start screen

    int ballLocationX = BALL_X_START;	// Starting x location of the ball
    int ballLocationY = BALL_Y_START;	// Starting y location of the ball
    int paddle1Y = 0;			// Starting y coord of left paddle
    int paddle2Y = 0;			// Starting y coord of right paddle
    
    //* -- Create the masks for the PIO lines used for the sensors
    u_int maskA = PA0 | PA1 | PA2 | PA3 | PA4 | PA5 | PA6 | PA8 | PA23 | PA20 | PA24 | PA25 | PA10 | PA11 | PA12 | PA13;
    u_int maskB = PB7 | PB2 | PB1 | PB0 | PB5 | PB4 | PB3 | PB16;

    at91_pio_open ( &PIOB_DESC, LED_MASK, PIO_OUTPUT ) ;
    at91_pio_write (&PIOB_DESC, LED_MASK, LED_OFF ) ;
    
    //* -- Set up PIOA and PIOB
    at91_pio_open ( &PIOA_DESC, maskA, PIO_INPUT ) ;
    at91_pio_write ( &PIOA_DESC, maskA, PIO_SET_OUT );
    at91_pio_open ( &PIOB_DESC, maskB, PIO_INPUT );
    at91_pio_write ( &PIOB_DESC, maskB, PIO_SET_OUT );

    // Enabling interrupts when programming with the slingshot
    EnableInterrupt();
    
    //* -- Set up serial
    serial.usart_desc = &USART_DESC;
    serial.term_data = &serialData;
    serial.baud_rate = (u_int) BAUDS38400;
    serial.format = (u_int) US_ASYNC_MODE;
    serial.terminal_asm_handler = at91_irq_handler;
    
    //* Open USART
    at91_terminal_open(&serial);
    
    //* USART IRQ initialisation //Note: I don't know what the magic numbers are
    at91_irq_open( USART_DESC.periph_id, 7, 0x40, at91_irq_handler );
    
    //* Enable RXRDY et TXRDY interrupt //Note: I don't know what the magic numbers are
    serial.usart_desc->usart_base->US_IER = 0x03;
    
    //* Detect Master Clock //We don't need this
    //wait_desc.mcki_khz = 32000 ;
	
    // Main loop!
    while( 1 )
    {
	// Check if we are allowed to poll for button presses yet
	// This is so the poll doens't happen so fast that one button press is
	// interpretted as two or more.
	if ( buttonWait > 0 )
	{
	    buttonWait--;
	}
	else
	{
	    buttonStatusA = at91_pio_read (&PIOA_DESC) ;
	    buttonStatusB = at91_pio_read (&PIOB_DESC) ;

	    // Button 1 is for pausing/unpausing the game.
	    if ( ( buttonStatusB & ( 1 << PIO_SW1 ) ) == 0 )
	    {
		if ( pause == PAUSED )
		{
		    SendInstructionsToFPGA( PAUSE_SCREEN, WRITE, 0 );
		    pause = NOT_PAUSED;
		    buttonWait = BUTTON_DELAY;
		}
		else
		{
		    SendInstructionsToFPGA( PAUSE_SCREEN, ERASE, 0 );
		    pause = PAUSED;
		    buttonWait = BUTTON_DELAY * 10;
		}
	    }
	    
	    // Button 2 is for resetting the score
	    if ( ( buttonStatusA & ( 1 << PIO_SW2 ) ) == 0 )
	    {
		player1Score = 0;
		player2Score = 0;
		buttonWait = BUTTON_DELAY;
	    }

	    // Button 3 is for enabling/disabling player1 AI
	    if ( ( buttonStatusB & ( 1 << PIO_SW3 ) ) == 0 )
	    {
		if( player1_AI == AI_ON )
		{
		    player1_AI = AI_OFF;
		}
		else
		{
		    player1_AI = AI_ON;
		}
		buttonWait = BUTTON_DELAY;
	    }

	    // Button 4 is for enabling/disabling player2 AI
	    if ( ( buttonStatusB & ( 1 << PIO_SW4 ) ) == 0 )
	    {
		if( player2_AI == AI_ON )
		{
		    player2_AI = AI_OFF;
		}
		else
		{
		    player2_AI = 1;
		}
		buttonWait = BUTTON_DELAY;
	    }
	}

	// If the game is paused, then just keep drawing the paddles and balls where they were
	// Otherwise, actually draw the stuff moving.
	if ( pause )
	{
	    SendInstructionsToFPGA( PADDLE1_ID, PADDLE1_X_OFFSET, paddle1Y );
	    SendInstructionsToFPGA( PADDLE2_ID, PADDLE2_X_OFFSET, paddle2Y );
	    SendInstructionsToFPGA( BALL_ID, ballLocationX, ballLocationY );
	    SendInstructionsToFPGA( PLAYER1_SCORE_ID, player1Score / 10 + 0x30, player1Score % 10 + 0x30 );
	    SendInstructionsToFPGA( PLAYER2_SCORE_ID, player2Score / 10 + 0x30, player2Score % 10 + 0x30 );   
	}
	else
	{
	    //See which sensors are triggered now
	    currentSensorsA = GetSensorStatusA();
	    currentSensorsB = GetSensorStatusB();
	
	    topMostSensorA = GetSensorNumber( currentSensorsA );
	    topMostSensorB = GetSensorNumber( currentSensorsB );

	    // Check if player1 is controlled by AI
	    // In either case, find out where to draw the paddle
	    if ( player1_AI )
	    {
		// Don't let the paddle move off the screen
		if( ballLocationY > 3 & ballLocationY < 60 )
		{
		    paddle1Y = ballLocationY - 4;
		}
	    }
	    else
	    {
		// Only move the paddle if there is a triggered sensor
		if ( topMostSensorA < 13 )
		{
		    paddle1Y = PIXEL_DISTANCE_BETWEEN_SENSORS * (topMostSensorA - 1);
		}
	    }

	    // Check if player2 is controlled by AI
	    // In either case, find out where to draw the paddle
	    if ( player2_AI )
	    {
		// Don't let the paddle move off the screen
		if ( ballLocationY > 3 && ballLocationY < 60 )
		{
		    paddle2Y = ballLocationY - 4;
		}
	    }
	    else
	    {
		// Only move the paddle if there is a triggered sensor
		//--------- HACK ---------//
		// Sensor 1 is always triggered (I think), so all other sensors have priority over it
		// The paddle is only redrawn for sensor 1 if it is not already there
		if( ( topMostSensorB > 1 && topMostSensorB < 13 ) || ( topMostSensorB == 1 && paddle2Y != 0 ) )
		{
		    paddle2Y = PIXEL_DISTANCE_BETWEEN_SENSORS * (topMostSensorB - 1);
		}
	    }

	    // Send packet serially
	    SendInstructionsToFPGA( PADDLE1_ID, PADDLE1_X_OFFSET, paddle1Y );
	    SendInstructionsToFPGA( PADDLE2_ID, PADDLE2_X_OFFSET, paddle2Y );

	    // Calculate the next ball location and see if it hit anything
	    hit = CalculateNextBallLocation( &ballLocationX, &ballLocationY, &ballDirectionX, &ballDirectionY,
		    &ballSpeedX, &ballSpeedY, paddle1Y, paddle2Y );

	    // Number of times to draw the ball (between 1 and 20 times
	    // This controls the speed of the ball
	    for( speedCounter = MAX_SPEED - speed + 1; speedCounter > 0; speedCounter-- )
	    {
		SendInstructionsToFPGA( BALL_ID, ballLocationX, ballLocationY );
	    }
	    
	    //Make sure score doesn't go over 100
	    if ( player1Score > MAX_SCORE )
	    {
		player1Score = 0;
	    }
	    if ( player2Score > MAX_SCORE )
	    {
		player2Score = 0;
	    }
	    
	    // Send the score for both players (convert the value to an ascii value)
	    SendInstructionsToFPGA( PLAYER1_SCORE_ID, player1Score / 10 + 0x30, player1Score % 10 + 0x30 );
	    SendInstructionsToFPGA( PLAYER2_SCORE_ID, player2Score / 10 + 0x30, player2Score % 10 + 0x30 );
	    
	    // Increase the speed of the ball if it hit a paddle and it isn't at max speed
	    // If one of the players scores, reset the ball and speed
	    if( speed < MAX_SPEED && hit == HIT_PADDLE )
	    {	
		speed++;
	    }
	    else if( hit == PLAYER1_SCORE )
	    {
		//Reset the speed and ball
		speed = MIN_SPEED;
		ballLocationX = BALL_X_START;
		ballLocationY = BALL_Y_START;
		ballDirectionX = LEFT;

		// Pseudo-randomization of ball starting y direction
		if( player1Score % 2 == 1 )
		{
		    ballDirectionY = UP;
		}
		else
		{
		    ballDirectionY = DOWN;
		}

		// Pause the game for a while between goals by drawing the ball repeatedly
		for( speedCounter = PAUSE_BETWEEN_GOALS; speedCounter > 0; speedCounter-- )
		{	
		    SendInstructionsToFPGA( BALL_ID, ballLocationX, ballLocationY );
		}
	    }
	    else if( hit == PLAYER2_SCORE )
	    {
		speed = MIN_SPEED;
		ballLocationX = BALL_X_START;
		ballLocationY = BALL_Y_START;
		ballDirectionX = RIGHT;

		// Pseudo-randomization of ball starting y direction
		if( player2Score % 2 == 1 )
		{
		    ballDirectionY = UP;
		}
		else
		{
		    ballDirectionY = DOWN;
		}

		// Pause the game for a while between goals by drawing the ball repeatedly
		for( speedCounter = PAUSE_BETWEEN_GOALS; speedCounter > 0; speedCounter-- )
		{	
		    SendInstructionsToFPGA( BALL_ID, ballLocationX, ballLocationY );
		}
	    }
	}
    }
    
#if 0 
    //* Close terminal 
    at91_terminal_close(&terminal_1); 
 
    //* End 
    return(0); 
#endif 
}
//* End

//*----------------------------------------------------------------------------
//* Function Name       : CalculateNextBallLocation
//* Object              : Calculate the position and direction of the ball
//* Input Parameters    : ball x, ball y, ball x direction, ball y direction,
//*			  left paddle y, right paddle y
//* Output Parameters   : none (values are returned in the first 4 param)
//*----------------------------------------------------------------------------
u_char CalculateNextBallLocation( int* ballLocationX, int* ballLocationY, u_char* ballDirectionX, u_char* ballDirectionY,
	u_char* ballSpeedX, u_char* ballSpeedY, int paddle1Y, int paddle2Y )
{
    u_char hit = HIT_NOTHING;
    
    // Control the X movement.
    if( *ballDirectionX == LEFT )
    {
	//If paddle1 is in the way, change direction
	if ( ( *ballLocationX - 1 ) == PADDLE1_X_OFFSET && ( *ballLocationY >= ( paddle1Y - 1 ) && *ballLocationY <= ( paddle1Y + PADDLE_LENGTH ) ) )
	{
	    *ballLocationX = PADDLE1_X_OFFSET * 2 + 2 - *ballLocationX;
	    *ballDirectionX = RIGHT;
	    hit = HIT_PADDLE;
	}
	//If we hit the left wall, change direction
	else if ( *ballLocationX - 1 < 0 )
	{
	    *ballLocationX = 1 - *ballLocationX;
	    *ballDirectionX  = RIGHT;
	    player2Score++;
	    hit = PLAYER2_SCORE;
	}
	//Otherwise keep going the same direction
	else
	{
	    *ballLocationX -= 1;
	}
    }
    else if ( *ballDirectionX == RIGHT )
    {
	//If paddle2 is in the way, change direction
	if ( ( *ballLocationX + 1 ) == PADDLE2_X_OFFSET && ( *ballLocationY >= ( paddle2Y - 1 ) && *ballLocationY <= ( paddle2Y + PADDLE_LENGTH ) ) )
	{
	    *ballLocationX = PADDLE2_X_OFFSET * 2 - 2 - *ballLocationX;
	    *ballDirectionX = LEFT;
	    hit = HIT_PADDLE;
	}
	//If we hit the right wall, change direction
	else if ( *ballLocationX + 2 > SCREEN_SIZE_X )
	{
	    *ballLocationX = SCREEN_SIZE_X * 2 - 2 - *ballLocationX;
	    *ballDirectionX = LEFT;
	    player1Score++;
	    hit = PLAYER1_SCORE;
	}
	//Otherwise keep going the same direction
	else
	{
	    *ballLocationX += 1;
	}
    }

    // Control the Y movement.
    if ( *ballDirectionY == UP )
    {
	//If we hit the top wall, change direction
	if ( *ballLocationY - 1 < 0 )
	{
	    *ballLocationY = 1 - *ballLocationY;
	    *ballDirectionY = DOWN;
	}
	//Otherwise keep going the same direction
	else
	{
	    *ballLocationY -= 1;
	}
    }
    else if ( *ballDirectionY == DOWN )
    {
	//If we hit the bottom wall, change direction
	if ( *ballLocationY + 2 > SCREEN_SIZE_Y )
	{
	    *ballLocationY = SCREEN_SIZE_Y * 2 - 2 - *ballLocationY;
	    *ballDirectionY = UP;
	}
	//Otherwise keep going the same direction
	else
	{
	    *ballLocationY += 1;
	}
    }

    return hit;
}

//*----------------------------------------------------------------------------
//* Function Name       : GetSensorNumber
//* Object              : Find the top most sensor we are at.
//* Input Parameters    : none
//* Output Parameters   : u_int - a value indicating the triggered sensors
//*----------------------------------------------------------------------------
u_int GetSensorNumber ( u_int triggeredSensors )
//* Begin
{
    u_int topMostSensor;
    
    if( ( triggeredSensors >> 0 ) & 1 == 1 )
    {
	topMostSensor = 12;
    }
    else if( ( triggeredSensors >> 1 ) & 1 == 1 )
    {
	topMostSensor = 11;
    }
    else if( ( triggeredSensors >> 2 ) & 1 == 1 )
    {
	topMostSensor = 10;
    }
    else if( ( triggeredSensors >> 3 ) & 1 == 1 )
    {
	topMostSensor = 9;
    }
    else if( ( triggeredSensors >> 4 ) & 1 == 1 )
    {
	topMostSensor = 8;
    }
    else if( ( triggeredSensors >> 5 ) & 1 == 1 )
    {
	topMostSensor = 7;
    }
    else if( ( triggeredSensors >> 6 ) & 1 == 1 )
    {
	topMostSensor = 6;
    }
    else if( ( triggeredSensors >> 7 ) & 1 == 1 )
    {
	topMostSensor = 5;
    }
    else if( ( triggeredSensors >> 8 ) & 1 == 1 )
    {
	topMostSensor = 4;
    }
    else if( ( triggeredSensors >> 9 ) & 1 == 1 )
    {
	topMostSensor = 3;
    }
    else if( ( triggeredSensors >> 10 ) & 1 == 1 )
    {
	topMostSensor = 2;
    }
    else if( ( triggeredSensors >> 11 ) & 1 == 1 )
    {
	topMostSensor = 1;
    }
    else
    {
	topMostSensor = 13;
    }

    return topMostSensor;
}
//* End


//*----------------------------------------------------------------------------
//* Function Name       : GetSensorStatusA
//* Object              : Indicate which sensors are triggered. This is a value
//*			  between 000 and FFF, where 0 means the motion sensor
//*			  is not triggered and 1 means it is.
//* Input Parameters    : none
//* Output Parameters   : u_int - a value indicating the triggered sensors
//*----------------------------------------------------------------------------
u_int GetSensorStatusA ( void )
//* Begin
{
    u_int status = at91_pio_read ( &PIOA_DESC );
    u_int sensorsTriggered = 0;

    if ( ( status & PA0 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 11);
    }
    if ( ( status & PA2 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 10);
    }
    if ( ( status & PA1 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 9);
    }	
    if ( ( status & PA4 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 8);
    }
    if ( ( status & PA3 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 7);
    }
    if ( ( status & PA6 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 6);
    }
    if ( ( status & PA5 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 5);
    }
    if ( ( status & PA8 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 4);
    }
    if ( ( status & PA25 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 3);
    }
    if ( ( status & PA24 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 2);
    }
    if ( ( status & PA23 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 1);
    }
    if ( ( status & PA20 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 0);
    }

    return sensorsTriggered;
}
//* End

//*----------------------------------------------------------------------------
//* Function Name       : GetSensorStatusB
//* Object              : Indicate which sensors are triggered. This is a value
//*			  between 000 and FFF, where 0 means the motion sensor
//*			  is not triggered and 1 means it is.
//* Input Parameters    : none
//* Output Parameters   : u_int - a value indicating the triggered sensors
//*----------------------------------------------------------------------------
u_int GetSensorStatusB ( void )
//* Begin
{
    u_int statusA = at91_pio_read ( &PIOA_DESC ) ;
    u_int statusB = at91_pio_read ( &PIOB_DESC ) ;
    u_int sensorsTriggered = 0;
    
    if ( ( statusB & PB7 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 11);
    }
    if ( ( statusB & PB2 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 10);
    }
    if ( ( statusB & PB1 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 9);
    }
    if ( ( statusB & PB0 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 8);
    }
    if ( ( statusB & PB5 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 7);
    }
    if ( ( statusB & PB4 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 6);
    }
    if ( ( statusB & PB3 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 5);
    }
    if ( ( statusA & PA13 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 4);
    }
    if ( ( statusA & PA12 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 3);
    }
    if ( ( statusA & PA11 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 2);
    }
    if ( ( statusA & PA10 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 1);
    }
    if ( ( statusB & PB16 ) == 0 )
    {
	sensorsTriggered = sensorsTriggered | (1 << 0);
    }

    return sensorsTriggered;
}
//* End

//*----------------------------------------------------------------------------
//* Function Name       : SendInstructionsToFPGA
//* Object              : Send a packet to the FPGA, telling it what to draw.
//* Input Parameters    : two u_int corresponding to the position of the users.
//* Output Parameters   : none
//*----------------------------------------------------------------------------
//* Packet:	start/stop identifier	2 bytes
//*		sprite ID		1 byte
//*		x-coord			2 bytes
//*		y-coord 		2 bytes
//*
//*		total			7 bytes
//*----------------------------------------------------------------------------
void SendInstructionsToFPGA ( u_char spriteId, u_short xCoord, u_short yCoord )
//* Begin
{
    char packet[5];

    packet[0] = BEGINNING_OF_PACKET;
    packet[1] = spriteId;
    packet[2] = (u_char)( xCoord );
    packet[3] = (u_char)( yCoord );
    packet[4] = END_OF_PACKET;
    at91_usart_send_frame( &USART_DESC, packet, 5 );
}
//* End
