/* Based off main.c for Lab #1
 * Nancy Minderman 
 * April 20, 2007
 */

/* This is going to be code for tracking the motion sensors.
 * Functions:
 * 	DetermineDirection
 *	EstimateVelocity
 *	GetSensorNumber
 *	GetSensorStatusA
 *	GetSensorStatusB
 */

#ifdef SEMIHOSTING
#include <stdio.h>
#endif

#include "parts/m55800/eb55.h"
#include "parts/m55800/lib_m55800.h"
#include "drivers/capture/capture.h"
#include "drivers/wait/wait.h"

#include "periph/stdc/std_c.h"
#include "drivers/terminal/terminal.h"
#include "periph/usart/usart.h"

/* Defines */

/* USART Stuff */
#define USART_DESC       USART2_DESC
#define BAUDS38400       (32000000 / (16 * 38400))    //* CD = 52

/* Ball stuff */
#define UP				1	//Direction of ball
#define DOWN				2	//Direction of ball
#define LEFT				3	//Direction of ball
#define RIGHT				4	//Direction of ball
#define BALL_X_OFFSET			1	//Size of the ball
#define BALL_Y_OFFSET			1	//Size of the ball
#define BALL_X_START			40	//Starting x coord of ball
#define BALL_Y_START			32	//Starting y coord of ball
#define HIT_NOTHING			0	//The ball hasn't hit anything
#define HIT_PADDLE			1	//The ball hit a paddle
#define PLAYER1_SCORE			2	//The ball hit the right side
#define PLAYER2_SCORE			3	//The ball hit the left side
#define MIN_SPEED			1	//The slowest speed for the ball
#define MAX_SPEED			20	//The fastest speed for the ball

/* Paddle stuff */
#define PADDLE_LENGTH			9	//The length of the paddle
#define PADDLE1_X_OFFSET		5	//The x coord of the left paddle
#define PADDLE2_X_OFFSET		75	//The x coord of the right paddle
#define PIXEL_DISTANCE_BETWEEN_SENSORS	5	//Used to calculate location of paddle

/* Game Stuff */
#define NOT_PAUSED			0	//The game is not paused
#define PAUSED				1	//The game is paused
#define AI_OFF				0	//The AI is on
#define AI_ON				1	//The AI is not on
#define BUTTON_DELAY			10	//Delay for polling button presses
#define MAX_SCORE			99	//The maximum score for the game
#define PAUSE_BETWEEN_GOALS		1000	//Give players a chance to rest between goals
#define SCREEN_SIZE_X			80	//Maximum x coord (far right)
#define SCREEN_SIZE_Y			64	//Maximum y coord (bottom)

/* Packet stuff */
#define BALL_ID				1	//Sprite ID for the ball
#define PADDLE1_ID			2	//Sprite ID for the left paddle
#define PADDLE2_ID			3	//Sprite ID for the right paddle
#define PLAYER1_SCORE_ID		4	//Sprite ID for the player1's score
#define PLAYER2_SCORE_ID		5	//Sprite ID for the player2's score
#define PAUSE_SCREEN			6	//ID for paused game
#define START_SCREEN			7	//Start Game
#define WRITE				1
#define ERASE				2
#define BEGINNING_OF_PACKET		0x24	//Beginning of packet
#define END_OF_PACKET			0x33	//End of packet

/* Functions */
extern void wake_up_handler (void) ;

//*----------------------------------------------------------------------------
//* Function Name       : CalculateNextBallLocation
//* Object              : Calculate the position and direction of the ball
//* Input Parameters    : ball x, ball y, ball x direction, ball y direction,
//*			  ball speed x, ball speed y left paddle y, right paddle y
//* Output Parameters   : a value corresponding to what the ball hit (see defines)
//*----------------------------------------------------------------------------
u_char CalculateNextBallLocation( int* ballLocationX, int* ballLocationY, u_char* directionX, u_char* directionY,
	u_char* ballSpeedX, u_char* ballSpeedY, int paddle1Y, int paddle2Y );

//*----------------------------------------------------------------------------
//* Function Name       : GetSensorNumber
//* Object              : Find the left most sensor we are at.
//* Input Parameters    : none
//* Output Parameters   : u_int - a value indicating the triggered sensors
//*----------------------------------------------------------------------------
u_int GetSensorNumber ( u_int triggeredSensors );

//*----------------------------------------------------------------------------
//* Function Name       : GetSensorStatusA
//* Object              : Indicate which sensors are triggered. This is a value
//*			  between 000 and FFF, where 0 means the motion sensor
//*			  is not triggered and 1 means it is.
//* Input Parameters    : none
//* Output Parameters   : u_int - a value indicating the triggered sensors
//*----------------------------------------------------------------------------
u_int GetSensorStatusA ( void );

//*----------------------------------------------------------------------------
//* Function Name       : GetSensorStatusB
//* Object              : Indicate which sensors are triggered. This is a value
//*			  between 000 and FFF, where 0 means the motion sensor
//*			  is not triggered and 1 means it is.
//* Input Parameters    : none
//* Output Parameters   : u_int - a value indicating the triggered sensors
//*----------------------------------------------------------------------------
u_int GetSensorStatusB ( void );

//*----------------------------------------------------------------------------
//* Function Name       : SendInstructionsToFPGA
//* Object              : Send a packet to the FPGA, telling it what to draw.
//* Input Parameters    : a sprite id and two shorts
//* Output Parameters   : none
//*----------------------------------------------------------------------------
//* For Sprite IDs 1-3:
//*
//* Packet:	start identifier	1 byte
//*		sprite ID		1 byte
//*		x-coord			1 byte
//*		y-coord 		1 byte
//*		stop idenitifier	1 byte
//*
//*		total			5 bytes
//*
//* For Sprite IDs 4-5:
//*
//* Packet:	start identifier	1 byte
//*		sprite ID		1 byte
//*		tens digit		1 byte
//*		ones digit 		1 byte
//*		stop idenitifier	1 byte
//*
//*		total			5 bytes
//*----------------------------------------------------------------------------
void SendInstructionsToFPGA ( u_char spriteId, u_short xCoord, u_short yCoord );

